/*
   +----------------------------------------------------------------------+
   | PHP version 4.0                                                      |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2001 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: David Eriksson <david@2good.com>                            |
   +----------------------------------------------------------------------+
 */

/*
 * $Id: zval_to_namedvalue.h,v 1.1.1.2 2001/07/19 00:20:01 zarzycki Exp $
 * vim: syntax=c tabstop=2 shiftwidth=2
 */

#ifndef __zval_to_namedvalue_h__
#define __zval_to_namedvalue_h__

#include <zend.h>
#include <orb/orbit.h>

zend_bool orbit_zval_to_namedvalue(const zval * pSource, CORBA_NamedValue * pDestination);

#endif /* __zval_to_namedvalue_h__ */

